@@ -1,4 +1,20 @@
-# Copyright Asitha Senanayake
+# Copyright (C) 2020  Asitha Senanayake
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+#
+# https://github.com/asitha-sena/python-ags4
+
 
 from pandas import DataFrame, to_numeric
 
@@ -227,6 +243,9 @@ def convert_to_text(dataframe, dictionary):
                     i = int(TYPE[0])
                     df.loc[0:, col] = df.loc[0:, col].apply(lambda x: f"{x:.{i}e}")
 
+                # TODO: Add formatting for data type 'nSF'
+                # elif 'SF' in TYPE:
+
                 df.loc[-2, col] = UNIT
                 df.loc[-1, col] = TYPE
 
@@ -234,3 +253,52 @@ def convert_to_text(dataframe, dictionary):
                 print(f"*WARNING*:{col} not found in the dictionary file.")
 
     return df.sort_index()
+
+
+def AGS4_to_excel(input_file, output_file, encoding='utf-8'):
+    """Load all the tables in a AGS4 file to an Excel spreasheet.
+
+    Input:
+    -----
+    input_file    - Path to AGS4 file
+    output_file   - Path to Excel file
+
+    Output:
+    ------
+    Excel file populated with data from the input AGS4 file.
+    """
+
+    from pandas import ExcelWriter
+
+    # Extract AGS4 file into a dictionary of dictionaries
+    tables, headings = AGS4_to_dataframe(input_file, encoding=encoding)
+
+    # Write to Excel file
+    with ExcelWriter(output_file) as writer:
+        for key in tables:
+            tables[key].to_excel(writer, sheet_name=key, index=False)
+
+
+def excel_to_AGS4(input_file, output_file):
+    """Export AGS4 data in Excel file to .ags file.
+
+    Input:
+    -----
+    input_file    - Path to Excel file
+                    (Note: Each GROUP should be in a separate worksheet.
+                     e.g. output from AGS4.AGS4_to_excel)
+    output_file   - Path to AGS4 file
+
+    Output:
+    ------
+    AGS4 file with data from the input Excel file.
+    """
+
+    from pandas import read_excel
+
+    # Read data from Excel file in to DataFrames
+    tables = read_excel(input_file, sheet_name=None, engine='openpyxl')
+
+    # Export dictionary of DataFrames to AGS4 file
+    # TODO: Add column formatting and ordering
+    dataframe_to_AGS4(tables, {}, output_file)
